// JavaScript code for the choropleth map with sliders
// Set up dimensions for the map container
const width = window.innerWidth * 0.75;
const height = window.innerHeight;

// Create SVG element for the map
const svg = d3
  .select('#map-container')
  .append('svg')
  .attr('width', '100%')
  .attr('height', height);

// Create tooltip
const tooltip = d3
  .select('body')
  .append('div')
  .attr('class', 'tooltip');

// Load world GeoJSON data
d3.json(
  'https://cdn.jsdelivr.net/npm/world-atlas/countries-110m.json',
).then(function (world) {
  // Convert TopoJSON to GeoJSON
  const countries = topojson.feature(
    world,
    world.objects.countries,
  );

  // Load data science salary data
  d3.csv(
    'https://gist.githubusercontent.com/SeshadriKamini7/14ed8d61ff06a071338bf120f64dfa78/raw/a72296da2c1beaf38523d4804b2665791cc41955/data_science_salaries.csv',
  ).then(function (salaryData) {
    // Merge salary data with GeoJSON features
    countries.features.forEach(function (country) {
      const countryName = country.properties.name;
      const countrySalaryData = salaryData.find(
        (d) => d.employee_residence === countryName,
      );
      if (countrySalaryData) {
        country.properties.salary =
          +countrySalaryData.salary_in_usd;
      } else {
        country.properties.salary = null; // Set salary to null for countries without data
      }
    });

    // Define color scale for salaries
    const colorScale = d3
      .scaleLinear()
      .domain([
        0,
        d3.max(salaryData, (d) => +d.salary_in_usd),
      ])
      .range([
        '#440154',
        '#3b528b',
        '#21918c',
        '#5ec962',
        '#fde725',
      ]);

    // Create the map projection
    const projection = d3
      .geoMercator()
      .fitSize([width, height], countries);

    // Create path generator
    const path = d3.geoPath().projection(projection);

    // Draw countries
    svg
      .selectAll('path')
      .data(countries.features)
      .enter()
      .append('path')
      .attr('d', path)
      .attr('fill', (d) =>
        d.properties.salary
          ? colorScale(d.properties.salary)
          : '#555',
      )
      .attr('stroke', '#fff')
      .attr('stroke-width', 0.5)
      .on('mouseover', function (event, d) {
        // Display tooltip with salary information
        tooltip
          .style('display', 'block')
          .html(
            `<strong>${d.properties.name}</strong><br>Salary: $${d.properties.salary ? d.properties.salary.toLocaleString() : 'N/A'}`,
          )
          .style('left', event.pageX + 10 + 'px')
          .style('top', event.pageY - 20 + 'px');
      })
      .on('mouseout', function () {
        // Hide tooltip on mouseout
        tooltip.style('display', 'none');
      });

    // Add zoom functionality
    svg.call(
      d3
        .zoom()
        .scaleExtent([1, 8])
        .on('zoom', function (event) {
          svg
            .selectAll('path')
            .attr('transform', event.transform);
        }),
    );

    // Add slider functionality
    const minSlider = document.getElementById(
      'minSalarySlider',
    );
    const maxSlider = document.getElementById(
      'maxSalarySlider',
    );

    minSlider.addEventListener('input', updateMap);
    maxSlider.addEventListener('input', updateMap);

    function updateMap() {
      const minSalary = +minSlider.value;
      const maxSalary = +maxSlider.value;

      // Update color scale domain
      colorScale.domain([minSalary, maxSalary]);

      // Update map colors
      svg
        .selectAll('path')
        .attr('fill', (d) =>
          d.properties.salary
            ? colorScale(d.properties.salary)
            : '#555',
        );
    }
  });
});
